Package: gWidgetstcltk
Version: 0.0-55.1
Title: Toolkit implementation of gWidgets for tcltk package
Author: John Verzani
Maintainer: ORPHANED
Depends: R (>= 2.14.0), methods, gWidgets(>= 0.0.51), tcltk(>= 2.7.0),
        digest
Suggests:
Description: Port of the gWidgets API to the tcltk package. Requires Tk 8.5 or greater.
License: GPL (>= 2)
URL: http://gwidgets.r-forge.r-project.org/
LazyLoad: yes
Packaged: 2020-04-29 05:48:22 UTC; ripley
NeedsCompilation: no
Repository: CRAN
Date/Publication: 2020-04-29 05:51:03 UTC
X-CRAN-Original-Maintainer: John Verzani <jverzani@gmail.com>
X-CRAN-Comment: Orphaned on 2020-04-29 as requires orphaned package
        'gWidgets' by the same non-maintainer.
